@@ -1,15 +1,15 @@
 ;;; pass.el --- Major mode for password-store.el -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2015-2019  Nicolas Petton & Damien Cassou
+;; Copyright (C) 2015-2023  Nicolas Petton & Damien Cassou
 
 ;; Author: Nicolas Petton <petton.nicolas@gmail.com>
 ;;         Damien Cassou <damien@cassou.me>
-;; Package-Version: 2.0
-;; Package-Revision: 35e3f86e9687
-;; GIT: https://github.com/NicolasPetton/pass
-;; Package-Requires: ((emacs "25") (password-store "2.1.0") (password-store-otp "0.1.5") (f "0.17"))
+;; Package-Version: 20250721.1935
+;; Package-Revision: 7651389c5291
+;; URL: https://github.com/NicolasPetton/pass
+;; Package-Requires: ((emacs "25.1") (password-store "1.7.4") (password-store-otp "0.1.5") (f "0.17"))
 ;; Created: 09 Jun 2015
-;; Keywords: password-store, password, keychain
+;; Keywords: tools, files
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 
 ;;; Code:
 (require 'password-store)
+(require 'password-store-otp)
 (require 'imenu)
 (require 'button)
 (require 'f)
@@ -49,6 +50,17 @@
   :group 'pass
   :type 'string)
 
+(defcustom pass-username-fallback-on-filename nil
+  "If non-nil use an entry's filename as a fallback for the username field."
+  :group 'pass
+  :type 'boolean)
+
+(defcustom pass-suppress-confirmations nil
+  "If non-nil suppress yes-or-no confirmations for pass commands.
+Confirmation is suppressed for `pass-edit', `pass-kill' and `pass-quit'."
+  :group 'pass
+  :type 'boolean)
+
 (defvar pass-buffer-name "*Password-Store*"
   "Name of the pass buffer.")
 
@@ -58,6 +70,7 @@
     (define-key map (kbd "p") #'pass-prev-entry)
     (define-key map (kbd "M-n") #'pass-next-directory)
     (define-key map (kbd "M-p") #'pass-prev-directory)
+    (define-key map (kbd "e") #'pass-edit)
     (define-key map (kbd "k") #'pass-kill)
     (define-key map (kbd "s") #'isearch-forward)
     (define-key map (kbd "?") #'describe-mode)
@@ -82,6 +95,10 @@
   "Face for displaying the header of the pass buffer."
   :group 'pass)
 
+(defface pass-mode-keybinding-face '((t :inherit font-lock-constant-face))
+  "Face for displaying pass key bindings."
+  :group 'pass)
+
 (defface pass-mode-entry-face '((t . ()))
   "Face for displaying pass entry names."
   :group 'pass)
@@ -93,7 +110,7 @@
   "Face for displaying password-store directory names."
   :group 'pass)
 
-(define-derived-mode pass-mode fundamental-mode "Password-Store"
+(define-derived-mode pass-mode special-mode "Password-Store"
   "Major mode for editing password-stores.
 
 \\{pass-mode-map}"
@@ -152,7 +169,8 @@ Similar to `save-excursion' but only restore the point."
 (defun pass-quit ()
   "Kill the buffer quitting the window."
   (interactive)
-  (when (y-or-n-p "Kill all pass entry buffers? ")
+  (when (or pass-suppress-confirmations
+             (y-or-n-p "Kill all pass entry buffers? "))
     (dolist (buf (buffer-list))
       (with-current-buffer buf
         (when (eq major-mode 'pass-view-mode)
@@ -221,11 +239,20 @@ all entries in the pass buffer."
     (password-store-rename entry new-name)
     (pass-update-buffer)))
 
+(defun pass-edit ()
+  "Edit the entry at point."
+  (interactive)
+  (pass--with-closest-entry entry
+    (when (or pass-suppress-confirmations
+               (yes-or-no-p (format "Do you want edit the entry %s? " entry)))
+      (password-store-edit entry))))
+
 (defun pass-kill ()
   "Remove the entry at point."
   (interactive)
   (pass--with-closest-entry entry
-    (when (yes-or-no-p (format "Do you want remove the entry %s? " entry))
+    (when (or pass-suppress-confirmations
+               (yes-or-no-p (format "Do you want remove the entry %s? " entry)))
       (password-store-remove entry)
       (pass-update-buffer))))
 
@@ -280,8 +307,9 @@ user input."
 (defun pass--copy-field (field)
   "Add FIELD of entry at point to kill ring."
   (pass--with-closest-entry entry
-    (let* ((inhibit-message t)
-           (parsed-entries (password-store-parse-entry entry)))
+    (let ((parsed-entries
+           (let ((inhibit-message t))
+             (password-store-parse-entry entry))))
       (unless (assoc field parsed-entries)
         (user-error "Field `%s' not in  %s" field entry))
       (password-store-copy-field entry field))))
@@ -298,9 +326,19 @@ using all fields in the entry."
     (pass--copy-field field)))
 
 (defun pass-copy-username ()
-  "Add username of entry at point to kill ring."
+  "Add username of entry at point to kill ring.
+
+If the entry does not have a username field/value within the entry, and if
+`pass-username-fallback-on-filename' is non-nil, copy the entry name instead."
   (interactive)
-  (pass--copy-field pass-username-field))
+  (condition-case err
+      (pass--copy-field pass-username-field)
+    (user-error
+     (if (not pass-username-fallback-on-filename)
+         (signal (car err) (cdr err))) ;; rethrow
+     (pass--with-closest-entry entry
+       (let ((entry-name (file-name-nondirectory entry)))
+         (password-store--save-field-in-kill-ring entry entry-name "username"))))))
 
 (defun pass-copy-url ()
   "Add url of entry at point to kill ring."
@@ -316,37 +354,55 @@ using all fields in the entry."
 (defun pass-display-header ()
   "Display the header in to the current buffer."
   (insert "Password-store directory:")
-  (put-text-property (point-at-bol) (point) 'face 'pass-mode-header-face)
+  (put-text-property (line-end-position) (point) 'face 'pass-mode-header-face)
   (insert " ")
   (pass--display-keybindings-toggle)
   (insert "\n\n")
   (when pass-show-keybindings
-    (pass--display-keybindings '((pass-copy . "Copy password")
-                                 (pass-copy-username . "Copy username")
-                                 (pass-copy-url . "Copy url")))
-    (insert "\n")
-    (pass--display-keybindings '((pass-copy-field . "Copy field")
-                                 (pass-goto-entry . "Jump to Entry")
-                                 (pass-browse-url . "Browse url")))
-                                 
-    (insert "\n")
-    (pass--display-keybindings '((pass-insert . "Insert")
-                                 (pass-next-entry . "Next")
-                                 (pass-update-buffer . "Update")))
-    (insert "\n")
-    (pass--display-keybindings '((pass-insert-generated . "Generate")
-                                 (pass-prev-entry . "Previous")
-                                 (pass-otp-options . "OTP Support")))
-    (insert "\n")
-    (pass--display-keybindings '((pass-rename . "Rename")
-                                 (pass-next-directory . "Next dir")
-                                 (pass-view . "View entry")))
-    (insert "\n")
-    (pass--display-keybindings '((pass-kill . "Delete")
-                                 (pass-prev-directory . "Previous dir")
-                                 (describe-mode . "Help")))
-    (newline)
-    (newline)))
+    (let* ((r1-bindings '((pass-copy . "Copy password")
+                          (pass-copy-username . "Copy username")
+                          (pass-copy-url . "Copy url")))
+           (r2-bindings '((pass-copy-field . "Copy field")
+                          (pass-goto-entry . "Jump to Entry")
+                          (pass-browse-url . "Browse url")))
+           (r3-bindings '((pass-insert . "Insert")
+                          (pass-next-entry . "Next")
+                          (pass-update-buffer . "Update")))
+           (r4-bindings '((pass-insert-generated . "Generate")
+                          (pass-prev-entry . "Previous")
+                          (pass-otp-options . "OTP Support")))
+           (r5-bindings '((pass-rename . "Rename")
+                          (pass-next-directory . "Next dir")
+                          (pass-view . "View entry")))
+           (r6-bindings '((pass-kill . "Delete")
+                          (pass-prev-directory . "Previous dir")
+                          (describe-mode . "Help")))
+           (r7-bindings '((pass-edit . "Edit")))
+           (all-bindings (append r1-bindings r2-bindings r3-bindings
+                                 r4-bindings r5-bindings r6-bindings
+                                 r7-bindings))
+           (command-lengths (mapcar (lambda (desc)
+                                      (length
+                                       (substitute-command-keys
+                                        (format "<\\[%s]>"
+                                                (symbol-name (car desc))))))
+                                    all-bindings))
+           (padding (1+ (apply #'max command-lengths))))
+      (pass--display-keybindings r1-bindings padding)
+      (insert "\n")
+      (pass--display-keybindings r2-bindings padding)
+      (insert "\n")
+      (pass--display-keybindings r3-bindings padding)
+      (insert "\n")
+      (pass--display-keybindings r4-bindings padding)
+      (insert "\n")
+      (pass--display-keybindings r5-bindings padding)
+      (insert "\n")
+      (pass--display-keybindings r6-bindings padding)
+      (insert "\n")
+      (pass--display-keybindings r7-bindings padding)
+      (newline)
+      (newline))))
 
 (defun pass--display-keybindings-toggle ()
   "Display a button to toggle whether keybindings should be displayed."
@@ -363,20 +419,23 @@ using all fields in the entry."
        (list (custom-quote (symbol-value pass-show-keybindings))))
   (pass-update-buffer))
 
-(defun pass--display-keybindings (bindings)
+(defun pass--display-keybindings (bindings padding)
   "Display the keybinding in each item of BINDINGS.
-BINDINGS is an alist of bindings."
+BINDINGS is an alist of bindings.
+PADDING is the padding used when inserting key strings."
   (mapc (lambda (pair)
-          (pass--display-keybinding (car pair) (cdr pair)))
+          (pass--display-keybinding (car pair) (cdr pair) padding))
         bindings))
 
-(defun pass--display-keybinding (command label)
-  "Insert the associated keybinding for COMMAND with LABEL."
-  (insert (format "%8s %-13s \t "
+(defun pass--display-keybinding (command label padding)
+  "Insert the associated keybinding for COMMAND with LABEL.
+
+PADDING is the padding used when inserting key strings."
+  (insert (format (concat "%" (number-to-string padding) "s %-13s \t ")
                   (format "%s"
                           (propertize (substitute-command-keys
                                        (format "<\\[%s]>" (symbol-name command)))
-                                      'face 'font-lock-constant-face))
+                                      'face 'pass-mode-keybinding-face))
                   label)))
 
 (defun pass-display-item (item &optional indent-level)
@@ -394,7 +453,7 @@ ITEM."
   "Display the password-store entry ENTRY into the current buffer."
   (let ((entry-name (f-filename entry)))
     (insert entry-name)
-    (add-text-properties (point-at-bol) (point)
+    (add-text-properties (line-beginning-position) (point)
                          `(face pass-mode-entry-face pass-entry ,entry))
     (newline)))
 
@@ -407,7 +466,7 @@ indented according to INDENT-LEVEL."
   (let ((name (car directory))
         (items (cdr directory)))
     (insert name)
-    (add-text-properties (point-at-bol) (point)
+    (add-text-properties (line-beginning-position) (point)
                          `(face pass-mode-directory-face pass-directory ,name))
     (newline)
     (dolist (item items)
@@ -422,7 +481,7 @@ indented according to INDENT-LEVEL."
 
 (defun pass-entry-at-point ()
   "Return the `pass-entry' property at point."
-  (get-text-property (point-at-eol) 'pass-entry))
+  (get-text-property (line-end-position) 'pass-entry))
 
 (defun pass-directory-at-point ()
   "Return the `pass-directory' property at point."
@@ -480,7 +539,7 @@ The password is read from user input."
   (pass-update-buffer))
 
 (defun pass-otp-from-screenshot ()
-  "Append an OTP URI taken from a screenshot to an existing entry in the password-store."
+  "Like `pass-otp-apped', but take the OTP URI from a screenshot."
   (interactive)
   (pass--with-closest-entry entry
     (password-store-otp-append-from-image entry))
@@ -538,14 +597,15 @@ This function only works when `pass-view-mode' is enabled."
 (defun pass-view-toggle-password ()
   "Enable or disable password hiding."
   (interactive)
-  (save-excursion
-    (goto-char (point-min))
-    (let ((buf-modified (buffer-modified-p)))
-      (if (string= (get-text-property (point) 'display)
-                   pass-view-mask)
-          (pass-view-unmask-password)
-        (pass-view-mask-password))
-      (set-buffer-modified-p buf-modified))))
+  (let (buffer-undo-list)
+    (save-excursion
+      (goto-char (point-min))
+      (let ((buf-modified (buffer-modified-p)))
+        (if (string= (get-text-property (point) 'display)
+                     pass-view-mask)
+            (pass-view-unmask-password)
+          (pass-view-mask-password))
+        (set-buffer-modified-p buf-modified)))))
 
 (defun pass-view-copy-password ()
   "Copy the password of the entry in the current buffer."
@@ -560,15 +620,17 @@ This function only works when `pass-view-mode' is enabled."
   (let ((inhibit-read-only t))
     (save-excursion
       (goto-char (point-min))
-      (set-text-properties (point-min) (line-end-position)
-                           `(display ,pass-view-mask)))))
+      (with-silent-modifications
+        (set-text-properties (point-min) (line-end-position)
+                             `(display ,pass-view-mask))))))
 
 (defun pass-view-unmask-password ()
   "Show the password in the current buffer."
   (save-excursion
     (goto-char (point-min))
-    (remove-text-properties (point-min) (line-end-position)
-                            '(display nil))))
+    (with-silent-modifications
+      (remove-text-properties (point-min) (line-end-position)
+                              '(display nil)))))
 
 (defun pass-view-copy-token ()
   "Copy current `pass-view' buffer's OTP token into clipboard."
@@ -606,7 +668,7 @@ This function only works when `pass-view-mode' is enabled."
                           token " - " remaining-secs "s remaining"))
         (key-binding (concat (propertize (substitute-command-keys
                                           (format "<\\[%s]>" "pass-view-copy-token"))
-                                         'face 'font-lock-constant-face)
+                                         'face 'pass-mode-keybinding-face)
                              " Copy token")))
     (setq header-line-format (concat otp-data "    " key-binding))
     (force-mode-line-update)))
@@ -619,7 +681,8 @@ nil otherwise."
     (search-forward "otpauth://" nil t)))
 
 (defun pass-view--otp-counter (buffer &optional last-token force-create)
-  "Reload BUFFER's OTP token and countdown, using LAST-TOKEN if any, and if FORCE-CREATE, build Header Line from scratch."
+  "Reload BUFFER's OTP token and countdown, using LAST-TOKEN if any.
+If FORCE-CREATE is non-nil, build Header Line from scratch."
   (when (buffer-live-p buffer)
     (with-current-buffer buffer
       (when (or header-line-format force-create)
@@ -657,7 +720,7 @@ This function also binds a couple of handy OTP related key-bindings to
   (pass--with-closest-entry entry
     (password-store-url entry)))
 
-(defvar pass-view-font-lock-keywords '("^[^\s\n]+:" . 'font-lock-keyword-face)
+(defvar pass-view-font-lock-keywords '("\\(^[^:\t\n]+:\\) " 1 'font-lock-keyword-face)
   "Font lock keywords for ‘pass-view-mode’.")
 
 (define-derived-mode pass-view-mode nil "Pass-View"
